@@ -204,9 +204,16 @@ gis_summary_page_shown (GisPage *page)
 
   if (!gis_driver_save_data (GIS_PAGE (page)->driver, &local_error))
     {
-      /* FIXME: This should probably be shown to the user and some options
-       * provided to them. */
       g_warning ("Error saving data: %s", local_error->message);
+
+      GtkWindow *parent = GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (page)));
+      GtkWidget *dialog = adw_message_dialog_new (parent,
+                                                  _("Setup Failed"),
+                                                  local_error->message);
+      adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "close", _("Close"));
+      /* FIXME: Provide some more options for debugging or recovery */
+
+      gtk_window_present (GTK_WINDOW (dialog));
     }
 
   gis_driver_get_user_permissions (GIS_PAGE (page)->driver,
